/*
 */

#include <stdio.h>

// #include "printf.h"
//#include "sleep.h"

#include "xparameters.h"

// #include "xfsbl_error.h"

#include "vdma.h"

XAxiVdma vdma;

XAxiVdma_DmaSetup VDMAOutCfg;
XAxiVdma_DmaSetup VDMAInCfg;

u32 vdma_version() {
	return XAxiVdma_GetVersion(&vdma);
}

int vdma_out_start() {
	   int Status;

	   // MM2S Startup
	   Status = XAxiVdma_DmaStart(&vdma, XAXIVDMA_READ);
	   if (Status != XST_SUCCESS)
	   {
	      printf("Start read transfer failed %d\n", Status);
	      return XST_FAILURE;
	   }

	   return XST_SUCCESS;
}

int vdma_in_start() {
	   int Status;

	   // MM2S Startup
	   Status = XAxiVdma_DmaStart(&vdma, XAXIVDMA_WRITE);
	   if (Status != XST_SUCCESS)
	   {
	      printf("Start read transfer failed %d\n", Status);
	      return XST_FAILURE;
	   }

	   return XST_SUCCESS;
}

int vdma_stop() {
	   XAxiVdma_DmaStop(&vdma, XAXIVDMA_READ);
	   return XST_SUCCESS;
}


// int vdma_common_init( unsigned short DeviceID )
int vdma_common_init( XAxiVdma_Config *Config )
{
    int status;
//     XAxiVdma_Config *Config;

    // get basic configuration from the xparameters
//     Config = XAxiVdma_LookupConfig( DeviceID );
//     if ( !Config ) {
//         printf("XAxiVdma_LookupConfig failure\r\n");
//         return XST_FAILURE;
//     }

    // Initialize DMA
    status = XAxiVdma_CfgInitialize(&vdma, Config, Config->BaseAddress);
    if (status != XST_SUCCESS) {
        printf("XAxiVdma_CfgInitialize failure\r\n");
        return XST_FAILURE;
    }

    //printf("0x%08X ... ", Config->BaseAddress);

#if (XPAR_VDMA_AXI_VDMA_0_INCLUDE_MM2S == 1)    
    XAxiVdma_DmaStop(&vdma, XAXIVDMA_READ);
    XAxiVdma_ChannelReset( &(vdma.ReadChannel) );
#endif
#if (XPAR_VDMA_AXI_VDMA_0_INCLUDE_S2MM == 1)    
    XAxiVdma_DmaStop(&vdma, XAXIVDMA_WRITE);
    XAxiVdma_ChannelReset( &(vdma.WriteChannel) );
#endif


    return XST_SUCCESS;
}

int vdma_out_init(int fb_base_address, int h_width, int v_width, int bpp)
{
  int Status;

  int i;
  u32 storage_size;
  UINTPTR addr;
//   u32 reg;


  VDMAOutCfg.EnableCircularBuf = 1;
  VDMAOutCfg.EnableFrameCounter = 0;
  VDMAOutCfg.FixedFrameStoreAddr = 0;

  VDMAOutCfg.EnableSync = 1;
  VDMAOutCfg.PointNum = 0;

  VDMAOutCfg.FrameDelay = 0;

  VDMAOutCfg.VertSizeInput = v_width;
  VDMAOutCfg.HoriSizeInput = h_width * bpp;
  VDMAOutCfg.Stride = VDMAOutCfg.HoriSizeInput;

  Status = XAxiVdma_DmaConfig(&vdma, XAXIVDMA_READ, &VDMAOutCfg);
  if (Status != XST_SUCCESS) {
      printf("Read channel config failed %d\r\n", Status);

      return XST_FAILURE;
  }

// 	VDMAOutCfg.FrameStoreStartAddr[0] = fb_base_address;
  addr = fb_base_address;
  storage_size = VDMAOutCfg.Stride * VDMAOutCfg.VertSizeInput;
  for(i = 0; i < vdma.MaxNumFrames; i++) {
      printf("\n    ");
      VDMAOutCfg.FrameStoreStartAddr[i] = addr;
      printf("FB[%d] address: 0x%08X", i, (unsigned int)addr);
      addr += storage_size;
  }
  printf("\n  ");

  Status = XAxiVdma_DmaSetBufferAddr(&vdma, XAXIVDMA_READ, VDMAOutCfg.FrameStoreStartAddr);
  if (Status != XST_SUCCESS) {
      printf("Read channel set buffer address failed %d\r\n", Status);
      return XST_FAILURE;
  }

  // enable internal genlock
//   addr = vdma.BaseAddr;
//   reg = *((volatile int *)(addr+XAXIVDMA_TX_OFFSET+XAXIVDMA_CR_OFFSET ));
//   reg |= XAXIVDMA_CR_GENLCK_SRC_MASK;
//   *((volatile int *)(addr+XAXIVDMA_TX_OFFSET+XAXIVDMA_CR_OFFSET )) = reg;

  Status = vdma_out_start();
  if (Status != XST_SUCCESS) {
        printf("error starting VDMA..!");
        return Status;
  }

  return XST_SUCCESS;

}

int vdma_in_init(int fb_base_address, int h_width, int v_width, int bpp)
{
  int Status;

  UINTPTR addr;
  int i;
  u32 storage_size;
  u32 reg;

  VDMAInCfg.EnableCircularBuf = 1;
  VDMAInCfg.EnableFrameCounter = 0;
  VDMAInCfg.FixedFrameStoreAddr = 0;

  VDMAInCfg.EnableSync = 1;
  VDMAInCfg.PointNum = 0;

  VDMAInCfg.FrameDelay = 0;

  VDMAInCfg.VertSizeInput = v_width;
  VDMAInCfg.HoriSizeInput = h_width * bpp;
  VDMAInCfg.Stride = VDMAInCfg.HoriSizeInput;

  Status = XAxiVdma_DmaConfig(&vdma, XAXIVDMA_WRITE, &VDMAInCfg);
  if (Status != XST_SUCCESS) {
      printf("Read channel config failed %d\r\n", Status);

      return XST_FAILURE;
  }

// 	VDMAInCfg.FrameStoreStartAddr[0] = fb_base_address;
  addr = fb_base_address;
  storage_size = VDMAInCfg.Stride * VDMAInCfg.VertSizeInput;
  for(i = 0; i < vdma.MaxNumFrames; i++) {
      VDMAInCfg.FrameStoreStartAddr[i] = addr;
      printf("\tFB[%d] address: 0x%08X\r\n", i, (unsigned int)addr);
      addr += storage_size;
  }

  Status = XAxiVdma_DmaSetBufferAddr(&vdma, XAXIVDMA_WRITE, VDMAInCfg.FrameStoreStartAddr);
  if (Status != XST_SUCCESS) {
      printf("Write channel set buffer address failed %d\r\n", Status);
      return XST_FAILURE;
  }

  // set sync on SOF (TUSER) in AXIS
  Status = XAxiVdma_FsyncSrcSelect(&vdma, XAXIVDMA_S2MM_TUSER_FSYNC, XAXIVDMA_WRITE);
  if (Status != XST_SUCCESS) {
      printf("Write channel set fsync failed %d\r\n", Status);
      return XST_FAILURE;
  }

  // enable internal genlock
  addr = vdma.BaseAddr;
  reg = *((volatile int *)(addr+XAXIVDMA_RX_OFFSET+XAXIVDMA_CR_OFFSET ));
  reg |= XAXIVDMA_CR_GENLCK_SRC_MASK;
  *((volatile int *)(addr+XAXIVDMA_RX_OFFSET+XAXIVDMA_CR_OFFSET )) = reg;

  Status = vdma_in_start();
  if (Status != XST_SUCCESS) {
        printf("error starting VDMA..!");
        return Status;
  }

  return XST_SUCCESS;

}

