/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: main.c
 * Date: 28.11.2016
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>, Zdenek Pohl <xpohl@utia.cas.cz>
 * Description:
 *  Basic sobel demo for linux OS
 *
 * Changelog:
 *
 *
 *
 ********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>

#include <stdbool.h>
#include <xparameters.h>
#include <string.h>
#include <frame_size.h>

unsigned short tmp_FB_in[NUMPADCOLS*NUMROWS];
unsigned short tmp_FB_in_prev[NUMPADCOLS*NUMROWS];
unsigned short tmp_FB_out[NUMPADCOLS*NUMROWS];

#include <optflow.h>

#define HW_OPTFLOW
#define advanceFStore(f)  {f = (f + 1) % vdma.fb_num;}	/* cyclic increment over available framebuffers */
#include "sds_lib.h"
unsigned long long clock_start, clock_end_processing, clock_end_frame;
float timestamp_clk_freq_hz;
#define TIME_STAMP_INIT    clock_start = sds_clock_counter();
#define TIME_STAMP_1  { clock_end_processing = sds_clock_counter();}
#define TIME_STAMP_2  { clock_end_frame = sds_clock_counter();}
//#define TIME_STAMP_PRINT {printf("Image processing time: %llu, Total FPS: %f \r\n", clock_end_processing-clock_start, (float)XPAR_PSU_CORTEXA53_0_TIMESTAMP_CLK_FREQ/(float)(clock_end_frame-clock_start)); }
#define TIME_STAMP_PRINT {printf("Image processing time: %llu, Total FPS: %f \r\n", clock_end_processing-clock_start, timestamp_clk_freq_hz/(float)(clock_end_frame-clock_start)); }


void optflow_processing(unsigned short *prev_in_buffer, unsigned short *in_buffer, unsigned short *out_buffer)
{
TIME_STAMP_2
TIME_STAMP_PRINT
TIME_STAMP_INIT

//	img_process(prev_in_buffer, in_buffer, out_buffer);
	fpga_optflow ((yuv_t *)prev_in_buffer, (yuv_t *) in_buffer, (yuv_t *) out_buffer,
		HEIGHT, WIDTH, NUMPADCOLS, 10.0f);

TIME_STAMP_1
}
// ***************************************************************************
// DDR3 base address
#define DDR_MEM_BASE  XPAR_DDR_MEM_BASEADDR


// ***************************************************************************
// Video Timing Controller VTC
#include "vtc.h"
#define VTC_DEV_ID  XPAR_HDMI_OUT_V_TC_0_DEVICE_ID

vtc_t vtc;
extern vtc_timing_t vtc_resolutions[VTC_RES_CNT];

// ***************************************************************************
// Video DMA - VDMA
#include "vdma.h"
#define VDMA_DEV_ID   XPAR_VDMA_AXI_VDMA_0_DEVICE_ID
#define VDMA_FB_NUM   XPAR_VDMA_AXI_VDMA_0_NUM_FSTORES

vdma_t  vdma;


#define MAX_RES_X       1920
#define MAX_RES_Y       1200          //1080
#define MAX_FRAME_SIZE  2048*4096     // max size of one frame for VDMA aligned to kB according to XAPP1205

#define FB_OFFSET       0x20000000  // 512 MB
// #define FB_OFFSET       0x10000000  // 256 MB
// #define FB_OFFSET       0x8000000   // 128 MB
#define FB_BASE         DDR_MEM_BASE + FB_OFFSET

UINTPTR vFrameStoreStartAddr[VDMA_FB_NUM];	/* table of virtual addresses to frames in framebuffer */

// Frame offset within the FB storage
#define OFFSET_EN_NONE  0
#define OFFSET_EN_ZERO  1
#define OFFSET_EN_LEFT  2
#define OFFSET_EN_RIGHT 3
#define OFFSET_EN_DOWN  4
#define OFFSET_EN_UP    5
u8 offset_en = OFFSET_EN_NONE;

// when the VDMA TX supports unaligned tranfers the minimal offset is 1
// otherwise the minimal offset has to be aligned to 64-bit tranfers: (OFFSET_STEP * pixel_bytes) = 64b
// OFFSET_STEP is defined in number of pixels
#if (XPAR_VDMA_AXI_VDMA_0_INCLUDE_MM2S_DRE == 1)
#   define OFFSET_STEP 1
#else
#   define OFFSET_STEP 4  // 4 * 2B_per_pixel = 64b
#endif


// ***************************************************************************
// AVNET HDMI IN - ADV7611
#include "adv7611_im.h"
extern u8             adv7611_im_edid[256];
adv7611_im_timing_t   adv7611_tm;
adv7611_im_hot_plug_t adv7611_hp;


// ***************************************************************************
// AVNET HDMI OUT - ADV7511
#include "adv7511_im.h"
adv7511_timing_t  adv7511_tm;


// ***************************************************************************
// programmable clock wizard
//#include "clkwiz/clkwiz.h"
//#define CLKWIZ_BASE   XPAR_CLK_GEN_CLKGEN_VARIABLE_BASEADDR
//clkwiz_t  clkwiz;


// ***************************************************************************
// AVNET I2C controller
#include "i2c_im.h"
#define IM_I2C_CTRL_BASE XPAR_FMC_IM_I2C_BASEADDR

UINTPTR im_i2c_ctrl_base;


// ***************************************************************************
// define black color in YCrCb 4:2:2 color space
#define PIX_Y_BLACK     0x00
#define PIX_CBCR_BLACK  0x80

#define PIX_Y_WHITE     0xFF
#define PIX_CBCR_WHITE  0x80

/*
 * set output video chain resolution
 */
int set_out_resolution(/*clkwiz_t *clkwiz,*/ vtc_t *vtc, vdma_t *vdma, adv7511_timing_t *tm, u8 sel)
{
    int status = 0;
  //  u8  clkwiz_sel;

    /*
     * Initialize VDMA TX
     */
    printf("Initialize VDMA ... ");
    status = vdma_tx_stop( &(vdma->inst) );
    if (status != XST_SUCCESS) {
        printf( "Cannot stop VDMA TX transfer ... FAILED\r\n" );
        return XST_FAILURE;
    }

    vdma->tx_timing.res.x = vtc_resolutions[sel].hav;
    vdma->tx_timing.res.y = vtc_resolutions[sel].vav;

    printf("TX Init ... ");
    status = vdma_tx_init( vdma );
    if (status != XST_SUCCESS) {
        printf("FAILED\n\r");
        return XST_FAILURE;
    } else {
        printf("OK\n\r");
    }

    /*
     * initialize HDMIO
     */
    printf("Initialize HDMI OUT - ADV7511 ... " );

    tm->hav = vtc_resolutions[sel].hav;
    tm->hfp = vtc_resolutions[sel].hfp;
    tm->hsw = vtc_resolutions[sel].hsw;
    tm->hbp = vtc_resolutions[sel].hbp;
    tm->hsp = vtc_resolutions[sel].hsp;

    tm->vav = vtc_resolutions[sel].vav;
    tm->vfp = vtc_resolutions[sel].vfp;
    tm->vsw = vtc_resolutions[sel].vsw;
    tm->vbp = vtc_resolutions[sel].vbp;
    tm->vsp = vtc_resolutions[sel].vsp;

    status = adv7511_im_init(im_i2c_ctrl_base, tm, ADV7511_IM_CLK_DELAY_NEG_0_8_NS, ADV7511_IM_HPD_TEST_DISABLED);
    if ( status != XST_SUCCESS ) {
        printf("FAILED\n\r");
        return XST_FAILURE;
    } else {
        printf("OK\n\r");
    }

    /*
     * initialize timing controller
     */
    printf("Initialize Timing Controller ... ");
    vtc->timing_sel = sel;
    vtc_init_gen( vtc );
    printf(" %s ", vtc_resolutions[sel].name);
    printf(" OK\n\r");

    return XST_SUCCESS;
}


/*
 * Checks the cable plugged event (HDMI input) and initialize HDMI input
 */
void check_hdmii_cable_plugged(adv7611_im_hot_plug_t *hp)
{
    if ( !hp->is_cable_detected && adv7611_im_cable_detect(im_i2c_ctrl_base, 0) ) {
        hp->is_cable_detected = 1;
        hp->is_video_valid = 0;
        hp->is_unplugged = 0;
        printf("HDMII cable plugged\n\r");
    }
}


/*
 * Checks the cable unplugged event (HDMI input)
 */
void check_hdmii_cable_unplugged(adv7611_im_hot_plug_t *hp, adv7611_im_timing_t *tm)
{
    if ( hp->is_cable_detected && !adv7611_im_cable_detect(im_i2c_ctrl_base, 0) ) {
        // cable unplugged (not powered)
        hp->is_cable_detected = 0;
        hp->is_video_valid = 0;
        hp->is_unplugged = 1;
        tm->locked = 0;
        printf("HDMII cable unplugged\n\r");
    }
}


/*
 * Detects change of incomig video signals
 */
void check_hdmii_resolution(adv7611_im_hot_plug_t *hp, vdma_t *vdma, adv7611_im_timing_t *tm)
{
    if (hp->is_video_valid && hp->is_cable_detected) {
        (void)adv7611_im_get_timing(im_i2c_ctrl_base, tm);
        if (
            (vdma->rx_timing.res.x  != tm->hav) ||  (vdma->rx_timing.res.y  != tm->vav) ||
            (vdma->rx_timing.hfp    != tm->hfp) ||  (vdma->rx_timing.vfp    != tm->vfp) ||
            (vdma->rx_timing.hsw    != tm->hsw) ||  (vdma->rx_timing.vsw    != tm->vsw) ||
            (vdma->rx_timing.hbp    != tm->hbp) ||  (vdma->rx_timing.vbp    != tm->vbp) ||
            (vdma->rx_timing.hsp    != tm->hsp) ||  (vdma->rx_timing.vsp    != tm->vsp)
        ) {
            hp->is_video_valid = 0;
        }
    }
}


/*
 * Set VDMA input according to currently detect input video signal parameters
 */
void set_valid_hdmii_video_signal(adv7611_im_hot_plug_t *hp, vdma_t *vdma, adv7611_im_timing_t *tm)
{
    int status;

    if (hp->is_cable_detected && !hp->is_video_valid) {

        (void)adv7611_im_get_timing(im_i2c_ctrl_base, tm);
        if ( !adv7611_im_cable_detect(im_i2c_ctrl_base, 0) ) return;
        adv7611_im_print_timing(tm);

        if (tm->is_interlaced) {
            printf("\n\r\tERROR : Interlaced video signal is not supported!\n\r");
            return;
        }

        if (tm->is_encrypted) {
            printf("\n\r\tERROR : HDCP encrypted video signal is not supported!\n\r");
            return;
        }


        printf("Initialize VDMA ... ");
        status = vdma_rx_stop( &(vdma->inst) );
        if (status != XST_SUCCESS) {
            printf( "Cannot stop VDMA RX transfer ... FAILED\r\n" );
            return;
        }

        // Clear frame stores
//        clear_fb_YCrCb( vdma );

        // updete VDMA parameters
        vdma->rx_timing.res.x  = tm->hav;
        vdma->rx_timing.hfp    = tm->hfp;
        vdma->rx_timing.hsw    = tm->hsw;
        vdma->rx_timing.hbp    = tm->hbp;
        vdma->rx_timing.hsp    = tm->hsp;

        vdma->rx_timing.res.y  = tm->vav;
        vdma->rx_timing.vfp    = tm->vfp;
        vdma->rx_timing.vsw    = tm->vsw;
        vdma->rx_timing.vbp    = tm->vbp;
        vdma->rx_timing.vsp    = tm->vsp;

        vdma->rx_pix_bytes     = 2;
        vdma->rx_offset.x      = 0;
        vdma->rx_offset.y      = 0;

        // initialize VDMA input
        printf("RX Init ... ");
        status = vdma_rx_init( vdma );
        if (status != XST_SUCCESS) {
            printf( "FAILED\r\n" );
            return;
        } else {
            printf( "OK\r\n" );
        }

        // resolution changes have impact on errors flags - clear them
        vdma_clr_errors( vdma );

        hp->is_video_valid = 1;

    }
}


/*
 * set output frame offset within the input frame dimensions
 */

void set_hdmio_offset(adv7611_im_hot_plug_t *hp, vdma_t *vdma, u8 *off_en)
{
    if (hp->is_video_valid && *off_en) {
        switch ( *off_en ) {
            case OFFSET_EN_ZERO :
                vdma->tx_offset.x = 0;
                vdma->tx_offset.y = 0;
                break;
            case OFFSET_EN_LEFT :
                if ((int)vdma->tx_offset.x < (int)(vdma->rx_timing.res.x - vdma->tx_timing.res.x)) vdma->tx_offset.x += OFFSET_STEP;
                break;
            case OFFSET_EN_RIGHT :
                if (vdma->tx_offset.x > 0) vdma->tx_offset.x -= OFFSET_STEP;
                break;
            case OFFSET_EN_DOWN :
                if ((int)vdma->tx_offset.y < (int)(vdma->rx_timing.res.y - vdma->tx_timing.res.y)) vdma->tx_offset.y++;
                break;
            case OFFSET_EN_UP :
                if (vdma->tx_offset.y > 0) vdma->tx_offset.y--;
                break;
            default : break;
        }

        *off_en = OFFSET_EN_NONE;

        (void)vdma_tx_init( vdma );

        printf("Frame offset: [%d, %d]\n\r", (unsigned int)vdma->tx_offset.x, (unsigned int)vdma->tx_offset.y);
    }
}

// --------------------------------------------------------------------------


#include <fcntl.h>
#include <sys/mman.h>

#define MAP_SIZE 4096UL
#define MAP_MASK (MAP_SIZE - 1)

typedef struct {
	void *base;
	void *dev_base;
} mem_map_t;

unsigned map(int memfd, off_t dev_base, mem_map_t *m) {

    // Map one page of memory into user space such that the device is in that page, but it may not
    // be at the start of the page.

    m->base = mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, memfd, dev_base & ~MAP_MASK);
        if (m->base == (void *) -1) {
        printf("Can't map the memory to user space.\n");
        return 1;
    }
    printf("Memory mapped at address %p.\n", m->base);

    // get the address of the device in user space which will be an offset from the base
    // that was mapped as memory is mapped at the start of a page

    m->dev_base = m->base + (dev_base & MAP_MASK);

    return 0;
}

/*
 * Framebuffer memory mapping (assumes each frame start aligned to 4096 bytes
 */
unsigned maps(int memfd, off_t base, mem_map_t *m, unsigned long map_size) {

    // Map one page of memory into user space such that the device is in that page, but it may not
    // be at the start of the page.

	if ((base % MAP_SIZE) != 0) {
		printf("Start of memory block must be aligned to 4096 bytes\r\n");
		return 1;
	}

    m->base = mmap(0, map_size, PROT_READ | PROT_WRITE, MAP_SHARED, memfd, base);
        if (m->base == (void *) -1) {
        printf("Can't map the memory to user space.\n");
        return 1;
    }
    printf("Memory mapped at address %p.\n", m->base);

    // get the address of the device in user space which will be an offset from the base
    // that was mapped as memory is mapped at the start of a page

    //m->dev_base = m->base + (dev_base & MAP_MASK);
    m->dev_base = m->base; //(void*)(char*)(m->base);

    return 0;
}
// --------------------------------------------------------------------------
int main()
{

    int status;

//     u32 timeout;

//     int i;
//     u8 *y;
//     u8 *crcb;


    printf("\n\r\n\r*****************************************************************************\n\r");
    printf(        "*  Signal Processing Dept., UTIA AV CR, v.v.i.                              *\n\r");
    printf(        "*  HDMI In/Out Application on AVNET FMC Board                               *\n\r");
    printf(        "*****************************************************************************\n\r\n\r");


    printf("Time measurement calibration (10s) .... \r\n");
    TIME_STAMP_INIT;
    sleep(10);
    TIME_STAMP_1;
    printf("Measured %llu timer ticks in interval\r\n", clock_end_processing - clock_start);
    timestamp_clk_freq_hz = (float)(clock_end_processing - clock_start)/10.0f;
    printf("Measured timestamp frequency: %f Hz\r\n", timestamp_clk_freq_hz);

    int memfd;
    memfd = open("/dev/mem", O_RDWR | O_SYNC);
        if (memfd == -1) {
        printf("Can't open /dev/mem.\n");
        exit(0);
    }
    printf("/dev/mem opened.\n");

    /*
     * set programmable clock wizard - HDMI out pixel clock
     */
#if (0)
    printf("Initialize Clock Wizard ... ");

    clkwiz.base_addr    = CLKWIZ_BASE;
    clkwiz.clk_in       = 50.0;         //50 MHz

    status = clkwiz_init( &clkwiz, CLKWIZ_148_5_MHZ );
    if ( status != XST_SUCCESS ) {
        printf("FAILED\r\n");
        return XST_FAILURE;
    }
    printf("OK\r\n");
#endif

    /*
	 * Map IIC memory to userspace
	 */
	mem_map_t iic_map;

	if (map(memfd, IM_I2C_CTRL_BASE, &iic_map)) {
	 printf("IIC mapping to userspace failed\r\n");
	 exit(1);
	}

	im_i2c_ctrl_base = (UINTPTR)iic_map.dev_base;

    /*
     * Initialize I2C for AVNET HDMII-HDMIO board
     */
    printf("Initialize I2C controller for AVNET ... " );
    status = i2c_im_init(im_i2c_ctrl_base);
    if ( status != XST_SUCCESS) {
        printf("FAILED\n\r");
        return XST_FAILURE;
    } else {
        printf("OK\n\r");
    }


    /*
     * Initialize HDMI Input - ADV 7611 on AVNET board
     */
    printf("Initialize HDMI IN - ADV7611 ... ");
    status = adv7611_im_init(im_i2c_ctrl_base, adv7611_im_edid, 0);
    if ( status != XST_SUCCESS ) {
        printf( "FAILED\n\r");
        return XST_FAILURE;
    }
    printf( "OK\n\r");


    /*
     * Initialize HDMI Output - ADV 7511 on AVNET board - 1920x1080p60
     */
    printf("Initialize HDMI OUT - ADV7511 ... " );
    adv7511_tm.hav = vtc_resolutions[VTC_1920x1080p60].hav;
    adv7511_tm.hfp = vtc_resolutions[VTC_1920x1080p60].hfp;
    adv7511_tm.hsw = vtc_resolutions[VTC_1920x1080p60].hsw;
    adv7511_tm.hbp = vtc_resolutions[VTC_1920x1080p60].hbp;
    adv7511_tm.hsp = vtc_resolutions[VTC_1920x1080p60].hsp;

    adv7511_tm.vav = vtc_resolutions[VTC_1920x1080p60].vav;
    adv7511_tm.vfp = vtc_resolutions[VTC_1920x1080p60].vfp;
    adv7511_tm.vsw = vtc_resolutions[VTC_1920x1080p60].vsw;
    adv7511_tm.vbp = vtc_resolutions[VTC_1920x1080p60].vbp;
    adv7511_tm.vsp = vtc_resolutions[VTC_1920x1080p60].vsp;

    status = adv7511_im_init(im_i2c_ctrl_base, &adv7511_tm, ADV7511_IM_CLK_DELAY_NEG_0_8_NS, ADV7511_IM_HPD_TEST_DISABLED);
    if ( status != XST_SUCCESS ) {
        printf("FAILED\n\r");
        return XST_FAILURE;
    } else {
        printf("OK\n\r");
    }

    /*
     * initialize timing controller
     */



    printf("Initialize Timing Controller ... ");

    XVtc_Config *vtc_config;

	// get basic configuration from the xparameters
	vtc_config = XVtc_LookupConfig( VTC_DEV_ID );
	if ( vtc_config == NULL ) {
		printf("VTC instance with ID %d not found ... ", VTC_DEV_ID);
		return XST_FAILURE;
	}

   /*
    * Map vtc memory to userspace
	*/
	mem_map_t vtc_map;

	if (map(memfd, vtc_config->BaseAddress, &vtc_map)) {
	 printf("VTC mapping to userspace failed\r\n");
	 exit(1);
	}

	vtc_config->BaseAddress = (UINTPTR)vtc_map.dev_base;


	// initialize VTC
	status = XVtc_CfgInitialize(&vtc.inst, vtc_config, vtc_config->BaseAddress);
	if (status != XST_SUCCESS) {
		printf( "Initialization failed %d ... ", status);
		return XST_FAILURE;
	}

    vtc.timing_sel = VTC_1920x1080p60;
    vtc_init_gen( &vtc );
    printf(" %s ", vtc_resolutions[VTC_1920x1080p60].name);
    printf(" OK\n\r");






    /*
     * initialize VDMA output to display FB
     */
    printf("Initialize VDMA ... ");
    vdma.dev_id   = VDMA_DEV_ID;
    vdma.fb_base  = FB_BASE;
    vdma.fb_num   = VDMA_FB_NUM;

    printf("Common Init ... ");
    vdma.storage_res.x = MAX_RES_X;  // always set to maximum
    vdma.storage_res.y = MAX_RES_Y;  // always set to maximum

    vdma.tx_pix_bytes = 2;   // set pixel size in bytes

    /*
	 * Mapping of vdma FB to userspace, vopras
	 */
	mem_map_t vdma_fb_map[VDMA_FB_NUM];
	unsigned storage_stride  = vdma_align_stride(vdma.storage_res.x, vdma.tx_pix_bytes /*fb.vdma_tx_bytes*/ );
	unsigned storage_size    = vdma_align_storage_size(vdma.storage_res.y, storage_stride);
	//unsigned storage_offset  = (offset.x * vdma.tx_pix_bytes /*fb.vdma_tx_bytes*/) + (storage_stride * offset.y);

	unsigned fb_idx;
	for (fb_idx = 0; fb_idx < VDMA_FB_NUM; fb_idx++) {
		if (maps(memfd, (off_t)FB_BASE + /*storage_offset + */fb_idx * storage_size, &vdma_fb_map[fb_idx], storage_size)) {
			printf("VDMA FB mapping to userspace failed\r\n");
			return 1;
		}
		vFrameStoreStartAddr[fb_idx] = (UINTPTR)vdma_fb_map[fb_idx].dev_base;
		sds_mmap((void *)((unsigned long)FB_BASE + /*storage_offset + */(unsigned long)fb_idx * (unsigned long)storage_size), storage_size, (void *)vFrameStoreStartAddr[fb_idx]);
	}


    XAxiVdma_Config *vdma_config;

    // get basic configuration from the xparameters
    vdma_config = XAxiVdma_LookupConfig( vdma.dev_id );
    if ( !vdma_config ) {
        printf( "VDMA instance with ID %d not found ... ", vdma.dev_id);
        return XST_FAILURE;
    }

    /*
	 * Map vdma memory to userspace
	 */
	mem_map_t vdma_map;

	if (map(memfd, vdma_config->BaseAddress, &vdma_map)) {
	 printf("VDMA mapping to userspace failed\r\n");
	 return XST_FAILURE;
	}

	vdma_config->BaseAddress = (UINTPTR)vdma_map.dev_base;

    // Initialize DMA
    status = XAxiVdma_CfgInitialize(&vdma.inst, vdma_config, vdma_config->BaseAddress);
    if (status != XST_SUCCESS) {
        printf( "Initialization failed %d ... ", status);
        return XST_FAILURE;
    }

    printf("Common Init OK\r\n");

    //printf("0x%08X ... ", Config->BaseAddress);

    XAxiVdma_ChannelReset( &(vdma.inst.ReadChannel) );
    XAxiVdma_ChannelReset( &(vdma.inst.WriteChannel) );



    printf("TX Init ... ");
    vdma.tx_pix_bytes     = 2;
    vdma.tx_timing.res.x  = vtc_resolutions[VTC_1920x1080p60].hav;
    vdma.tx_timing.res.y  = vtc_resolutions[VTC_1920x1080p60].vav;
    vdma.tx_offset.x      = 0;
    vdma.tx_offset.y      = 0;

    status = vdma_tx_init( &vdma );
    if (status != XST_SUCCESS) {
        printf("FAILED\n\r");
        return XST_FAILURE;
    } else {
        printf("OK\n\r");
    }


    /*
     * clear HDMII flags and params before autodetection
     */
    adv7611_hp.is_cable_detected = 0;
    adv7611_hp.is_video_valid = 0;
    adv7611_hp.is_unplugged = 0;

    memset((void *)&adv7611_tm, 0, sizeof(adv7611_im_timing_t));

//    clear_fb_YCrCb( &vdma );


    /*
     * Check input cable plugged
     */
    check_hdmii_cable_plugged(&adv7611_hp);
    /*
     * Check input cable unplugged
     */
    check_hdmii_cable_unplugged(&adv7611_hp, &adv7611_tm);
    /*
     * Try to detect runtime resolution change of HDMI input
     */
    check_hdmii_resolution(&adv7611_hp, &vdma, &adv7611_tm);
    /*
     * configure VDMA
     */
    set_valid_hdmii_video_signal(&adv7611_hp, &vdma, &adv7611_tm);
    /*
     * set offset
     */
    set_hdmio_offset(&adv7611_hp, &vdma, &offset_en);


    //***********************************************************************************
       /*
         * Check available number of framebuffers
         */
      if (VDMA_FB_NUM >= 8) {
/*
		 * Setup VDMA parking mode
		 */
		printf("Starting VDMA parking mode ...\r\n");

		/*
		 * After some time (some = 1 sec)
		 */
	    sleep(1);



		/*
		 * start parking and define framebuffer ids
		 */
		unsigned writingFrame 		= 7;                        // frame ktery se zapisuje z hdmi vstupu
		unsigned nextInputFrame 	= 6; 						// frame ktery je pripraven az se zacne zpracovavat
		unsigned inputFrame   		= 5; 						// frame ktery se prave zpracovava
		unsigned previousInputFrame     = 4;
		unsigned nextOutputFrame  	= 3;			            // frame pripraveny na dalsi vystup
		unsigned outputFrame      	= 2;					 	// frame do ktereho se zapisuje vysledek zpracovani
		unsigned readingFrame     	= 1;		                // frame ktery se cte do hdmi vystupu
		unsigned rwIsolationFrame 	= 0;						// frame separating reading and writing frame

		/*
		 * Clear any error flags in VDMA
		 */
		vdma_clr_errors(&vdma);

		/*
		 * Start parking mode for READ and WRITE channels
		 */
		XAxiVdma_StartParking(&vdma.inst, writingFrame, XAXIVDMA_WRITE);vdma_clr_errors(&vdma);
		XAxiVdma_StartParking(&vdma.inst, readingFrame, XAXIVDMA_READ);vdma_clr_errors(&vdma);

		printf("Parking started\r\n");
		vdma_clr_errors(&vdma);   									/* clear errors on VDMA again if any */


//#define MAX_FPS

#ifdef MAX_FPS
		/*
		 * wait until newly set parking frames are reached by VDMA
		 */
		while (readingFrame != XAxiVdma_CurrFrameStore(&fb.vdma_hdmi, XAXIVDMA_READ)) {};
		while (writingFrame != XAxiVdma_CurrFrameStore(&fb.vdma_hdmi, XAXIVDMA_WRITE)) {};
#endif

		int strip_pos = 0;
		int strip_step = 1;
		int strip_pos_min = 0;
		int strip_pos_max = 1080 - HEIGHT;


		while (1) {
#ifndef MAX_FPS
			/*
			 * wait until newly set parking frames are reached by VDMA
			 */
			while (readingFrame != XAxiVdma_CurrFrameStore(&vdma.inst, XAXIVDMA_READ)) {};
			while (writingFrame != XAxiVdma_CurrFrameStore(&vdma.inst, XAXIVDMA_WRITE)) {};

			/*
			 * Advance frame indexes
			 */
			advanceFStore(writingFrame);
			advanceFStore(nextInputFrame);
			advanceFStore(inputFrame);
			advanceFStore(previousInputFrame);
			advanceFStore(nextOutputFrame);
			advanceFStore(outputFrame);
			advanceFStore(readingFrame);

			/*
			 * Initiate parking to new positions (moved by one)
			 */
			XAxiVdma_StartParking(&vdma.inst, readingFrame, XAXIVDMA_READ);  // set parking to next, do not wait until switch
			XAxiVdma_StartParking(&vdma.inst, writingFrame, XAXIVDMA_WRITE); // set parking to next, do not wait until switch
#endif
			/*
			 * Do some processing on frames
			 */
#ifdef HW_OPTFLOW
         		optflow_processing((unsigned short*)vFrameStoreStartAddr[previousInputFrame] + strip_pos * NUMPADCOLS,
         						   (unsigned short*)vFrameStoreStartAddr[inputFrame] + strip_pos * NUMPADCOLS,
              				       (unsigned short*)vFrameStoreStartAddr[outputFrame] + strip_pos * NUMPADCOLS);
#else
			memcpy((void*)tmp_FB_in_prev, (void *)((unsigned short*)vFrameStoreStartAddr[previousInputFrame] + strip_pos * NUMPADCOLS), HEIGHT*NUMPADCOLS*sizeof(short) );
			memcpy((void*)tmp_FB_in, (void *)((unsigned short*)vFrameStoreStartAddr[inputFrame] + strip_pos * NUMPADCOLS), HEIGHT*NUMPADCOLS*sizeof(short) );
			optflow_processing(tmp_FB_in_prev,tmp_FB_in, tmp_FB_out);
			memcpy((void *)((unsigned short*)vFrameStoreStartAddr[outputFrame] + strip_pos * NUMPADCOLS),(void*)tmp_FB_out, HEIGHT*NUMPADCOLS*sizeof(short) );
#endif

         		if (strip_pos_max != strip_pos_min) strip_pos += strip_step;
         		if (strip_pos == strip_pos_max) strip_step = -1;
         		if (strip_pos == strip_pos_min) strip_step = 1;
               		}
	} else {
		printf("ERROR; not enough frame buffers to run sobel demo\r\n");
	}


   /*
     *  unmap the memory before exiting
     */
      if (munmap(iic_map.base, MAP_SIZE) == -1) {
          printf("Can't unmap memory from user space.\n");
          exit(0);
      }

      if (munmap(vtc_map.base, MAP_SIZE) == -1) {
          printf("Can't unmap memory from user space.\n");
          exit(0);
      }

      for (fb_idx = 0; fb_idx < VDMA_FB_NUM; fb_idx++) {
        if (munmap(vdma_fb_map[fb_idx].base, MAP_SIZE) == -1) {
            printf("Can't unmap memory from user space.\n");
            exit(0);
        }
       }

      if (munmap(vdma_map.base, MAP_SIZE) == -1) {
          printf("Can't unmap memory from user space.\n");
          exit(0);
      }

      close(memfd);

      printf("Memory device closed\r\n");
      printf("Exiting ...\r\n");

    exit(0);
}


























