/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: adv7511_im_im.h
 * Date: 16.09.2016
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>, Zdenek Pohl <xpohl@utia.cas.cz>
 * Description:
 *    Avnet FMC HDMI OUT (ADV7511) interface
 *
 * Changelog:
 *   xpohl, aarch64 compatibility
 *
 *
 *
 ********************************************/


#ifndef _ADV7511_IM_HEADER_FILE_
#define _ADV7511_IM_HEADER_FILE_


#define I2C_ADV7511_IM_BASE_ADDR        0x39  //0x72

#define I2C_ADV7511_IM_MUX_ADDR         0x70  // 0xE0/0xE1 (PCA9546A)
#define I2C_ADV7511_IM_MUX_DATA         0x02  // ADV7511
#define I2C_IO_EXP_IM_MUX_DATA          0x08  // PCA9555 IO Expander

#define I2C_EXP_ADV7511_IM_ADDR         0x20  // 0x40/0x41 (PCA9534)
#define IO_EXP_ADV7511_IM_PD_REG        0x01
#define IO_EXP_ADV7511_IM_PD_DATA       0x10

#define ADV7511_IM_EDID_I2C_ADDR_REG    0x43
#define ADV7511_IM_EDID_INT_EN_REG      0x94
#define ADV7511_IM_EDID_INT_REG         0x96
#define ADV7511_IM_EDID_REREAD_REG      0xC9
#define ADV7511_IM_EDID_SEG_REG         0xC4

#define ADV7511_IM_EDID_INT_EN_MASK         0x04
#define ADV7511_IM_EDID_RDY_MASK            0x04
#define ADV7511_IM_EDID_REREAD_MASK         0x10
#define ADV7511_IM_EDID_HPD_INT_MASK        0x80
#define ADV7511_IM_EDID_HPD_MON_SENSE_MASK  0x40

#define ADV7511_IM_HPD                      (ADV7511_IM_EDID_HPD_INT_MASK | ADV7511_IM_EDID_HPD_MON_SENSE_MASK)
#define ADV7511_IM_HPD_CLR                  0x20

#define ADV7511_IM_HPD_TEST_ENABLED         1
#define ADV7511_IM_HPD_TEST_DISABLED        0

#define ADV7511_IM_CLK_DELAY_REG            0xBA
#define ADV7511_IM_CLK_DELAY_NEG_1_2_NS     0x00
#define ADV7511_IM_CLK_DELAY_NEG_0_8_NS     0x20
#define ADV7511_IM_CLK_DELAY_NEG_0_4_NS     0x40
#define ADV7511_IM_CLK_DELAY_0_0_NS         0x60
#define ADV7511_IM_CLK_DELAY_0_4_NS         0x80
#define ADV7511_IM_CLK_DELAY_0_8_NS         0xA0
#define ADV7511_IM_CLK_DELAY_1_2_NS         0xC0
#define ADV7511_IM_CLK_DELAY_1_6_NS         0xE0

typedef struct adv7511_im_i2c
{
    char  rw;
    u8    reg;
    u8    data;
} adv7511_im_i2c_t;


typedef struct {
    // horizontal timing
    u32 hav;  // active video
    u32 hfp;  // front porch
    u32 hsw;  // sync width
    u32 hbp;  // back porch
    u32 hsp;  // sync polarity

    // vertical timing
    u32 vav;  // active video
    u32 vfp;  // front porch
    u32 vsw;  // sync width
    u32 vbp;  // back porch
    u32 vsp;  // sync polarity
} adv7511_timing_t;

#ifdef __cplusplus
extern "C" {
#endif

int adv7511_im_i2c_write(UINTPTR i2c_ctrl_base, u8 dev_addr, u8 reg, u8 data);
int adv7511_im_i2c_read(UINTPTR i2c_ctrl_base, u8 dev_addr, u8 reg, u8 print_en);

int adv7511_im_init_sequence(UINTPTR i2c_ctrl_base, u16 dev_addr, const adv7511_im_i2c_t conf[]);
void adv7511_calc_embedded_syncs(adv7511_timing_t *tm, adv7511_im_i2c_t conf[]);
int adv7511_im_pd(UINTPTR i2c_ctrl_base, u8 power_down );
int adv7511_im_adjust_clkin_delay(UINTPTR i2c_ctrl_base, u8 delay);

int adv7511_im_init(UINTPTR i2c_ctrl_base, adv7511_timing_t *timing, u8 clkin_delay, u8 hpd_test);

#ifdef __cplusplus
};
#endif


#endif
