/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: adv7611_im_im.h
 * Date: 15.09.2016
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>, Zdenek Pohl <xpohl@utia.cas.cz> *
 * Description:
 *    Avnet FMC HDMI IN (ADV7611) interface
 *
 * Changelog:
 * aarch64 compatibility
 *
 *
 *
 ********************************************/


#ifndef _ADV7611_IM_HEADER_FILE_
#define _ADV7611_IM_HEADER_FILE_

#define I2C_ADV7611_IM_MUX_ADDR             0x70 // 0xE0/0xE1 (PCA9546A)
#define I2C_ADV7611_IM_MUX_DATA             0x04 // ADV7611
#define I2C_IO_EXP_IM_MUX_DATA              0x08 // PCA9555 IO Expander

#define I2C_IO_EXP_IM_BASE_ADDR             0x20  // 0x40/0x41


#define I2C_ADV7611_IM_BASE_ADDR            0x98  // IO
#define I2C_ADV7611_IM_CEC_ADDR             0x80
#define I2C_ADV7611_IM_INFOFRAME_ADDR       0x7C
#define I2C_ADV7611_IM_DPLL_ADDR            0x4C
#define I2C_ADV7611_IM_KSV_ADDR             0x64
#define I2C_ADV7611_IM_EDID_ADDR            0x6C
#define I2C_ADV7611_IM_HDMI_ADDR            0x68
#define I2C_ADV7611_IM_CP_ADDR              0x44

#define ADV7611_IM_RSTN                     0x01

#define ADV7611_IM_REG_VS_INF_PB_0_28       0x6f
#define ADV7611_IM_CABLE_DET_A_RAW          0x01


// video signal locked
#define ADV7611_IM_REG_IO_6A                0x6A  // IO
#define ADV7611_IM_DE_REGEN_LCK_RAW         0x01  // IO, Address 0x6A[0]
#define ADV7611_IM_V_LOCKED_RAW             0x02  // IO, Address 0x6A[1]
#define ADV7611_IM_PLL_LOCKED_RAW           0x40  // IO, Address 0x6A[6]

#define ADV7611_IM_REG_HDMI_07              0x07  // HDMI
#define ADV7611_IM_DE_REGEN_FILTER_LOCKED   0x20  // Addr 68 (HDMI), Address 0x07[5]
#define ADV7611_IM_VERT_FILTER_LOCKED       0x80  // Addr 68 (HDMI), Address 0x07[7]

#define ADV7611_IM_REG_HDMI_04              0x04  // HDMI
#define ADV7611_IM_DE_TMDS_PLL_LOCKED       0x01  // Addr 68 (HDMI), Address 0x04[1]

#define ADV7611_IM_LOCKED                   (ADV7611_IM_DE_TMDS_PLL_LOCKED | ADV7611_IM_DE_REGEN_FILTER_LOCKED | ADV7611_IM_VERT_FILTER_LOCKED)


typedef struct adv7611_im_i2c
{
    char  rw;       // 'r', 'w'
    u8    space;    // address space
    u8    reg;      // register address within the adress space
    u8    data;     // data to write
} adv7611_im_i2c_t;


typedef struct adv7611_im_timing
{
    u32   hav;          // number of active pixels in a line
    u32   hfp;          // horizonatal front porch
    u32   hsw;          // horizonatal sync pulse width
    u32   hbp;          // horizonatal back porch
    u32   hsp;          // horizonatal sync pulse polarity, 0 - nagative, 1 - positive

    u32   vav;          // number of lines
    u32   vfp;          // vertical front porch
    u32   vsw;          // vertical sync pulse width
    u32   vbp;          // vertical back porch
    u32   vsp;          // vertical sync pulse polarity, 0 - nagative, 1 - positive

    float tmds_freq;    // pixel clock

    u32 is_hdmi;        // 0 - DVI, 1 - HDMI
    u32 is_encrypted;
    u32 is_interlaced;  // 0 - progressive, 1 - interlaced
    u32 color_depth;

    u32 locked;         // incoming video signal is locked
} adv7611_im_timing_t;


// Simple hot plug of HDMII
typedef struct {
    u8 is_cable_detected;
    u8 is_video_valid;
    u8 is_unplugged;
} adv7611_im_hot_plug_t;

#ifdef __cplusplus
extern "C" {
#endif

int adv7611_im_i2c_write(UINTPTR i2c_ctrl_base, u8 dev_addr, u8 reg, u8 data);
int adv7611_im_i2c_read(UINTPTR i2c_ctrl_base, u8 dev_addr, u8 reg, u8 print_en);

int adv7611_im_cable_detect(UINTPTR i2c_ctrl_base, u8 debug_print);
int adv7611_im_init(UINTPTR i2c_ctrl_base, u8 edid[256], u8 debug_print);
int adv7611_im_get_lock(UINTPTR i2c_ctrl_base);
int adv7611_im_get_timing(UINTPTR i2c_ctrl_base, adv7611_im_timing_t *tm);
void adv7611_im_print_timing(adv7611_im_timing_t *tm);

#ifdef __cplusplus
};
#endif




#endif
