/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: adv7511_te.c
 * Date: 09.12.2015
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *    Trenz TE0701-05 HDMI OUT (ADV7511) interface
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/

#include <stdio.h>
#include <string.h>
#include <xil_io.h>
#include <sleep.h>

#include <xiic.h>
#include <xiic_l.h>

#include "xparameters.h"

#include "xfsbl_error.h"

#include "adv7511_te.h"


// ***************************************************************************
const adv7511_te_i2c_t adv7511_te_dvi_init_conf[] =
{

    {'w', 0xD6, 0xC0},  // HPD is always high

    {'w', 0x15, 0xF5}, // ID 5
    {'w', 0x16, 0x38}, // 8, style 1, falling

    {'w', 0x48, 0x08}, // align right, right justified, DDR low bits, normal bus order


    {'w', 0x55, 0x12}, // AVI AVI valid, RGB, PC
    {'w', 0x56, 0x28}, //
    {'w', 0x57, 0xC8}, // 08
    {'w', 0x59, 0x10}, //


    {'w', 0x17, 0x00}, // DE gen disabled, aspec 4:3

    {'w', 0xD0, 0x3C}, // Disable DDR Negative Edge CLK Delay
                       // delay = 0.0ns 

//     {'w', 0xAF, 0x04}, // no HDCP
    {'w', 0xAF, 0x06}, // HDMI + HDCP

    // Conversion matrix IN=OUT
    //     {'w', 0x18, 0xA8},
    {'w', 0x18, 0x08},
    {'w', 0x19, 0x00},
    {'w', 0x1A, 0x00},
    {'w', 0x1B, 0x00},
    {'w', 0x1C, 0x00},
    {'w', 0x1D, 0x00},
    {'w', 0x1E, 0x00},
    {'w', 0x1F, 0x00},
    {'w', 0x20, 0x00},
    {'w', 0x21, 0x00},
    {'w', 0x22, 0x08},
    {'w', 0x23, 0x00},
    {'w', 0x24, 0x00},
    {'w', 0x25, 0x00},
    {'w', 0x26, 0x00},
    {'w', 0x27, 0x00},
    {'w', 0x28, 0x00},
    {'w', 0x29, 0x00},
    {'w', 0x2A, 0x00},
    {'w', 0x2B, 0x00},
    {'w', 0x2C, 0x08},
    {'w', 0x2D, 0x00},
    {'w', 0x2E, 0x00},
    {'w', 0x2F, 0x00},
    // ------------------------

    {'w', 0x41, 0x10},

    {'w', 0x98, 0x03}, // FIXED 0x03
    {'w', 0x9A, 0xE0}, // FIXED 0xE0
    {'w', 0x9C, 0x30}, // FIXED 0x30 PLL
    {'w', 0x9D, 0x61}, // xxxx_xx01 CLK Div
    {'w', 0xA2, 0xA4}, // FIXED
    {'w', 0xA3, 0xA4}, // FIXED

    {'w', 0xE0, 0xD0}, // FIXED
    {'w', 0xF9, 0x00},

    {'w', 0xFF, 0xFF},

    {'w', 0xBA, 0x60}, // no delay

    {0,0,0}
};



// ***************************************************************************


/*
 * I2C write ADV7511 register
 */
int adv7511_te_i2c_write(UINTPTR i2c_ctrl_base, u8 dev_addr, u8 reg, u8 data)
{
    u8 buff[2];

    buff[0] = reg;
    buff[1] = data;

    /*
     * Make sure all the Fifo's are cleared and Bus is Not busy.
     */
    while (((XIic_ReadReg(i2c_ctrl_base,
        XIIC_SR_REG_OFFSET)) & (XIIC_SR_RX_FIFO_EMPTY_MASK
        | XIIC_SR_TX_FIFO_EMPTY_MASK | XIIC_SR_BUS_BUSY_MASK))
        != (XIIC_SR_RX_FIFO_EMPTY_MASK | XIIC_SR_TX_FIFO_EMPTY_MASK)) {
    }

    if (XIic_DynSend(i2c_ctrl_base, dev_addr, buff, 2, XIIC_STOP) != 2)
        return -1;

    return 0;
}

/*
 * I2C read ADV7511 register
 */
int adv7511_te_i2c_read(UINTPTR i2c_ctrl_base, u8 dev_addr, u8 reg, u8 print_en)
{
    u8 buff[1] = {1};

    /*
     * Make sure all the Fifo's are cleared and Bus is Not busy.
     */
    while (((XIic_ReadReg(i2c_ctrl_base,
        XIIC_SR_REG_OFFSET)) & (XIIC_SR_RX_FIFO_EMPTY_MASK
        | XIIC_SR_TX_FIFO_EMPTY_MASK | XIIC_SR_BUS_BUSY_MASK))
        != (XIIC_SR_RX_FIFO_EMPTY_MASK | XIIC_SR_TX_FIFO_EMPTY_MASK)) {
    }

    if (XIic_DynSend(i2c_ctrl_base, dev_addr, &reg, 1, XIIC_REPEATED_START) != 1)
        return -1;

    if (XIic_DynRecv(i2c_ctrl_base, dev_addr, buff, 1) != 1)
        return -2;

    if (print_en)
        xil_printf("\tadv7511_te_i2c_read: reg %02x data %02x \n\r", reg, (u32)*buff);

    return (int)*buff;
}


/*
 * configure ADV7511 chip
 */
int adv7511_te_init_sequence(UINTPTR i2c_ctrl_base, u16 dev_addr, const adv7511_te_i2c_t conf[])
{
    u32 i = 0;
    int status;

    while(conf[i].rw) {
        switch(conf[i].rw) {
            case 'r':
                status = adv7511_te_i2c_read(i2c_ctrl_base, dev_addr, conf[i].reg, conf[i].data);
                if (status < 0) return -1;
                break;
            case 'w':
                status = adv7511_te_i2c_write(i2c_ctrl_base, dev_addr, conf[i].reg, conf[i].data);
                if (status < 0) return -2;
                break;
            default:
                break;
        }
        i++;
    }
    return 0;
}


/*
 * initialize I2C controller for ADV7511
 */
int adv7511_te_i2c_init(UINTPTR i2c_ctrl_base)
{
    int status = 0;

    /*
    * initialize i2c controller
    */
    status = XIic_DynInit(i2c_ctrl_base);
    if (status != XFSBL_SUCCESS) {
        return XFSBL_ERROR_I2C_INIT;
    }

    return XFSBL_SUCCESS;
}


/*
 * adjust input clock delay (predefined values in its header)
 */
int adv7511_te_adjust_clkin_delay(UINTPTR i2c_ctrl_base, u8 delay)
{
    int status;
    u8 val;

    // set input clock delay
    status = adv7511_te_i2c_read(i2c_ctrl_base, I2C_ADV7511_TE_ADDR, ADV7511_TE_CLK_DELAY_REG, 0);
    if ( status < 0 ) {
        return XFSBL_FAILURE;
    }

    val = (u8)status & 0x1F;
    val |= delay;
    
    status = adv7511_te_i2c_write(i2c_ctrl_base, I2C_ADV7511_TE_ADDR, ADV7511_TE_CLK_DELAY_REG, val);                   
    if ( status < 0 ) {
        return XFSBL_FAILURE;
    } 

    // Adjust for the Input Video CLK Negative Edge for DDR Capture

    return XFSBL_SUCCESS;
}


/*
 * initialize ADV7511 HDMI
 */
int adv7511_te_init(UINTPTR i2c_ctrl_base, u8 clkin_delay, u8 hpd_test)
{
    int status;
    int timeout;


    /*
     * wait for HPD interrupt
     */
    if ( hpd_test ) {
        timeout = 100;
        status = 0;
        while ( ( status != ADV7511_TE_HPD ) && ( --timeout ) ) {
            status = adv7511_te_i2c_read(i2c_ctrl_base, I2C_ADV7511_TE_ADDR, ADV7511_TE_EDID_INT_REG, 0) & ADV7511_TE_HPD;
            if ( status != ADV7511_TE_HPD ) usleep(10);
        }
        if ( !timeout ) {
            xil_printf("HDMI cable is unplugged ... ");
            return XFSBL_FAILURE;
        }
    }

    // configure ADV7511
    status = adv7511_te_init_sequence(i2c_ctrl_base, I2C_ADV7511_TE_ADDR, adv7511_te_dvi_init_conf);
    if (status != XFSBL_SUCCESS) {
        xil_printf("Cannot configure ... ");
        return XFSBL_ERROR_I2C_WRITE;
    }


    /*
     * adjust input clock delay (predefined values in its header)
     */
    status = adv7511_te_adjust_clkin_delay(i2c_ctrl_base, clkin_delay);
    if (status != XFSBL_SUCCESS) {
        xil_printf("Cannot adjust inpt clock delay ... ");
        return XFSBL_ERROR_I2C_WRITE;
    } 

    // chip revision
    xil_printf("rev. 0x%02X ... ", adv7511_te_i2c_read(i2c_ctrl_base, I2C_ADV7511_TE_ADDR, 0x00, 0));


    return XFSBL_SUCCESS;
}


